@@ -1,3 +1,4 @@
+# pylint: disable=too-many-lines
 import base64
 import datetime
 import http
@@ -35,9 +36,23 @@ from hypothesis import event, note, reject
 from hypothesis import strategies as st
 from starlette.testclient import TestClient as ASGIClient
 
-from . import serializers
-from .constants import SERIALIZERS_SUGGESTION_MESSAGE, USER_AGENT, CodeSampleStyle, DataGenerationMethod
-from .exceptions import CheckFailed, InvalidSchema, SerializationNotPossible, get_grouped_exception
+from . import failures, serializers
+from .constants import (
+    DEFAULT_RESPONSE_TIMEOUT,
+    SERIALIZERS_SUGGESTION_MESSAGE,
+    USER_AGENT,
+    CodeSampleStyle,
+    DataGenerationMethod,
+)
+from .exceptions import (
+    CheckFailed,
+    FailureContext,
+    InvalidSchema,
+    SerializationNotPossible,
+    UsageError,
+    get_grouped_exception,
+    get_timeout_error,
+)
 from .hooks import GLOBAL_HOOK_DISPATCHER, HookContext, HookDispatcher
 from .parameters import Parameter, ParameterSet, PayloadAlternatives
 from .serializers import Serializer, SerializerContext
@@ -81,6 +96,8 @@ class Case:  # pylint: disable=too-many-public-methods
     source: Optional[CaseSource] = attr.ib(default=None)  # pragma: no mutate
     # The media type for cases with a payload. For example, "application/json"
     media_type: Optional[str] = attr.ib(default=None)  # pragma: no mutate
+    # The way the case was generated (None for manually crafted ones)
+    data_generation_method: Optional[DataGenerationMethod] = attr.ib(default=None)  # pragma: no mutate
 
     def __repr__(self) -> str:
         parts = [f"{self.__class__.__name__}("]
@@ -255,7 +272,8 @@ class Case:  # pylint: disable=too-many-public-methods
         final_headers = self._get_headers(headers)
         if self.media_type and self.media_type != "multipart/form-data" and self.body is not NOT_SET:
             # `requests` will handle multipart form headers with the proper `boundary` value.
-            final_headers["Content-Type"] = self.media_type
+            if "content-type" not in {header.lower() for header in final_headers}:
+                final_headers["Content-Type"] = self.media_type
         base_url = self._get_base_url(base_url)
         formatted_path = self.formatted_path.lstrip("/")  # pragma: no mutate
         url = unquote(urljoin(base_url + "/", quote(formatted_path)))
@@ -266,6 +284,12 @@ class Case:  # pylint: disable=too-many-public-methods
             extra = serializer.as_requests(context, self.body)
         else:
             extra = {}
+        additional_headers = extra.pop("headers", None)
+        if additional_headers:
+            # Additional headers, needed for the serializer
+            for key, value in additional_headers.items():
+                if key.lower() not in {header.lower() for header in final_headers}:
+                    final_headers[key] = value
         return {
             "method": self.method,
             "url": url,
@@ -290,7 +314,16 @@ class Case:  # pylint: disable=too-many-public-methods
             close_session = False
         data = self.as_requests_kwargs(base_url, headers)
         data.update(kwargs)
-        response = session.request(**data)  # type: ignore
+        data.setdefault("timeout", DEFAULT_RESPONSE_TIMEOUT / 1000)
+        try:
+            response = session.request(**data)  # type: ignore
+        except requests.Timeout as exc:
+            timeout = 1000 * data["timeout"]  # It is defined and not empty, since the exception happened
+            code_message = self._get_code_message(self.operation.schema.code_sample_style, exc.request)
+            raise get_timeout_error(timeout)(
+                f"\n\n1. Request timed out after {timeout:.2f}ms\n\n----------\n\n{code_message}",
+                context=failures.RequestTimeout(timeout=timeout),
+            ) from None
         if close_session:
             session.close()
         return response
@@ -382,19 +415,21 @@ class Case:  # pylint: disable=too-many-public-methods
                 if code_sample_style is not None
                 else self.operation.schema.code_sample_style
             )
-            if code_sample_style == CodeSampleStyle.python:
-                code = self.get_code_to_reproduce(request=response.request)
-                code_message = f"Run this Python code to reproduce this response: \n\n    {code}\n"
-            elif code_sample_style == CodeSampleStyle.curl:
-                code = self.as_curl_command(headers=dict(response.request.headers))
-                code_message = f"Run this cURL command to reproduce this response: \n\n    {code}\n"
-            else:  # pragma: no cover
-                raise ValueError(f"Unknown code sample style: {code_sample_style.name}")
+            code_message = self._get_code_message(code_sample_style, response.request)
             payload = get_response_payload(response)
             raise exception_cls(
                 f"\n\n{formatted_errors}\n\n----------\n\nResponse payload: `{payload}`\n\n{code_message}"
             )
 
+    def _get_code_message(self, code_sample_style: CodeSampleStyle, request: requests.PreparedRequest) -> str:
+        if code_sample_style == CodeSampleStyle.python:
+            code = self.get_code_to_reproduce(request=request)
+            return f"Run this Python code to reproduce this response: \n\n    {code}\n"
+        if code_sample_style == CodeSampleStyle.curl:
+            code = self.as_curl_command(headers=dict(request.headers))
+            return f"Run this cURL command to reproduce this response: \n\n    {code}\n"
+        raise ValueError(f"Unknown code sample style: {code_sample_style.name}")
+
     def call_and_validate(
         self,
         base_url: Optional[str] = None,
@@ -403,9 +438,10 @@ class Case:  # pylint: disable=too-many-public-methods
         checks: Tuple["CheckFunction", ...] = (),
         code_sample_style: Optional[str] = None,
         **kwargs: Any,
-    ) -> None:
+    ) -> requests.Response:
         response = self.call(base_url, session, headers, **kwargs)
         self.validate_response(response, checks, code_sample_style=code_sample_style)
+        return response
 
     def get_full_url(self) -> str:
         """Make a full URL to the current API operation, including query parameters."""
@@ -418,6 +454,7 @@ class Case:  # pylint: disable=too-many-public-methods
     def partial_deepcopy(self) -> "Case":
         return self.__class__(
             operation=self.operation.partial_deepcopy(),
+            data_generation_method=self.data_generation_method,
             path_parameters=deepcopy(self.path_parameters),
             headers=deepcopy(self.headers),
             cookies=deepcopy(self.cookies),
@@ -606,7 +643,23 @@ class APIOperation(Generic[P, C]):
         body: Union[Body, NotSet] = NOT_SET,
         media_type: Optional[str] = None,
     ) -> C:
-        """Create a new example for this API operation."""
+        """Create a new example for this API operation.
+
+        The main use case is constructing Case instances completely manually, without data generation.
+        """
+        if body is not NOT_SET and media_type is None:
+            # If the user wants to send payload, then there should be a media type, otherwise the payload is ignored
+            media_types = self.get_request_payload_content_types()
+            if len(media_types) == 1:
+                # The only available option
+                media_type = media_types[0]
+            else:
+                media_types_repr = ", ".join(media_types)
+                raise UsageError(
+                    "Can not detect appropriate media type. "
+                    "You can either specify one of the defined media types "
+                    f"or pass any other media type available for serialization. Defined media types: {media_types_repr}"
+                )
         return self.case_cls(
             operation=self,
             path_parameters=path_parameters,
@@ -656,10 +709,13 @@ class Check:
 
     name: str = attr.ib()  # pragma: no mutate
     value: Status = attr.ib()  # pragma: no mutate
-    response: GenericResponse = attr.ib()  # pragma: no mutate
+    response: Optional[GenericResponse] = attr.ib()  # pragma: no mutate
     elapsed: float = attr.ib()  # pragma: no mutate
     example: Case = attr.ib()  # pragma: no mutate
     message: Optional[str] = attr.ib(default=None)  # pragma: no mutate
+    # Failure-specific context
+    context: Optional[FailureContext] = attr.ib(default=None)  # pragma: no mutate
+    request: Optional[requests.PreparedRequest] = attr.ib(default=None)  # pragma: no mutate
 
 
 @attr.s(slots=True, repr=False)  # pragma: no mutate
@@ -807,8 +863,11 @@ class Interaction:
 class TestResult:
     """Result of a single test."""
 
+    __test__ = False
+
     method: str = attr.ib()  # pragma: no mutate
     path: str = attr.ib()  # pragma: no mutate
+    verbose_name: str = attr.ib()  # pragma: no mutate
     data_generation_method: DataGenerationMethod = attr.ib()  # pragma: no mutate
     checks: List[Check] = attr.ib(factory=list)  # pragma: no mutate
     errors: List[Tuple[Exception, Optional[Case]]] = attr.ib(factory=list)  # pragma: no mutate
@@ -835,13 +894,35 @@ class TestResult:
     def has_logs(self) -> bool:
         return bool(self.logs)
 
-    def add_success(self, name: str, example: Case, response: GenericResponse, elapsed: float) -> None:
-        self.checks.append(Check(name=name, value=Status.success, response=response, elapsed=elapsed, example=example))
-
-    def add_failure(self, name: str, example: Case, response: GenericResponse, elapsed: float, message: str) -> None:
-        self.checks.append(
-            Check(name=name, value=Status.failure, response=response, elapsed=elapsed, example=example, message=message)
+    def add_success(self, name: str, example: Case, response: GenericResponse, elapsed: float) -> Check:
+        check = Check(
+            name=name, value=Status.success, response=response, elapsed=elapsed, example=example, request=None
         )
+        self.checks.append(check)
+        return check
+
+    def add_failure(
+        self,
+        name: str,
+        example: Case,
+        response: Optional[GenericResponse],
+        elapsed: float,
+        message: str,
+        context: Optional[FailureContext],
+        request: Optional[requests.PreparedRequest] = None,
+    ) -> Check:
+        check = Check(
+            name=name,
+            value=Status.failure,
+            response=response,
+            elapsed=elapsed,
+            example=example,
+            message=message,
+            context=context,
+            request=request,
+        )
+        self.checks.append(check)
+        return check
 
     def add_error(self, exception: Exception, example: Optional[Case] = None) -> None:
         self.errors.append((exception, example))
@@ -865,6 +946,8 @@ class TestResult:
 class TestResultSet:
     """Set of multiple test results."""
 
+    __test__ = False
+
     results: List[TestResult] = attr.ib(factory=list)  # pragma: no mutate
     generic_errors: List[InvalidSchema] = attr.ib(factory=list)  # pragma: no mutate
 
