@@ -1,6 +1,6 @@
 import threading
 import time
-from typing import Dict, List, Optional, Union
+from typing import Any, Dict, List, Optional, Union
 
 import attr
 from requests import exceptions
@@ -16,13 +16,21 @@ from .serialization import SerializedError, SerializedTestResult
 class ExecutionEvent:
     """Generic execution event."""
 
-    asdict = attr.asdict
+    # Whether this event is expected to be the last one in the event stream
+    is_terminal = False
+
+    def asdict(self, **kwargs: Any) -> Dict[str, Any]:
+        data = attr.asdict(self, **kwargs)
+        # An internal tag for simpler type identification
+        data["event_type"] = self.__class__.__name__
+        return data
 
 
 @attr.s(slots=True)  # pragma: no mutate
 class Initialized(ExecutionEvent):
     """Runner is initialized, settings are prepared, requests session is ready."""
 
+    schema: Dict[str, Any] = attr.ib()  # pragma: no mutate
     # Total number of operations in the schema
     operations_count: Optional[int] = attr.ib()  # pragma: no mutate
     # The place, where the API schema is located
@@ -39,6 +47,7 @@ class Initialized(ExecutionEvent):
     def from_schema(cls, *, schema: BaseSchema, count_operations: bool = True) -> "Initialized":
         """Computes all needed data from a schema instance."""
         return cls(
+            schema=schema.raw_schema,
             operations_count=schema.operations_count if count_operations else None,
             location=schema.location,
             base_url=schema.get_base_url(),
@@ -140,6 +149,8 @@ class Interrupted(ExecutionEvent):
 class InternalError(ExecutionEvent):
     """An error that happened inside the runner."""
 
+    is_terminal = True
+
     message: str = attr.ib()  # pragma: no mutate
     exception_type: str = attr.ib()  # pragma: no mutate
     exception: Optional[str] = attr.ib(default=None)  # pragma: no mutate
@@ -176,6 +187,8 @@ class Finished(ExecutionEvent):
     No more events after this point.
     """
 
+    is_terminal = True
+
     passed_count: int = attr.ib()  # pragma: no mutate
     failed_count: int = attr.ib()  # pragma: no mutate
     errored_count: int = attr.ib()  # pragma: no mutate
