@@ -1,5 +1,6 @@
 import json
 import re
+import string
 from base64 import b64encode
 from contextlib import contextmanager, suppress
 from copy import deepcopy
@@ -16,15 +17,17 @@ from ...constants import DataGenerationMethod
 from ...exceptions import InvalidSchema
 from ...hooks import GLOBAL_HOOK_DISPATCHER, HookContext, HookDispatcher
 from ...models import APIOperation, Case
-from ...schemas import BaseSchema
 from ...types import NotSet
-from ...utils import NOT_SET
+from ...utils import NOT_SET, compose
 from .constants import LOCATION_TO_CONTAINER
-from .parameters import OpenAPIParameter, parameters_to_json_schema
+from .negative import negative_schema
+from .parameters import OpenAPIBody, parameters_to_json_schema
+from .utils import is_header_location
 
 PARAMETERS = frozenset(("path_parameters", "headers", "cookies", "query", "body"))
 SLASH = "/"
 STRING_FORMATS = {}
+StrategyFactory = Callable[[Dict[str, Any], str, str, Optional[str]], st.SearchStrategy]
 
 
 def register_string_format(name: str, strategy: st.SearchStrategy) -> None:
@@ -51,13 +54,19 @@ def init_default_strategies() -> None:
 
     latin1_text = st.text(alphabet=st.characters(min_codepoint=0, max_codepoint=255))
 
+    # RFC 7230, Section 3.2.6
+    register_string_format(
+        "_header_name",
+        st.text(min_size=1, alphabet=st.sampled_from("!#$%&'*+-.^_`|~" + string.digits + string.ascii_letters)),
+    )
+    # Define valid characters here to avoid filtering them out in `is_valid_header` later
+    header_value = st.text(alphabet=st.characters(min_codepoint=0, max_codepoint=255, blacklist_characters="\n\r"))
+    # Header values with leading non-visible chars can't be sent with `requests`
+    register_string_format("_header_value", header_value.map(str.lstrip))
     register_string_format("_basic_auth", st.tuples(latin1_text, latin1_text).map(make_basic_auth_str))  # type: ignore
     register_string_format(
         "_bearer_auth",
-        # Define valid characters here to avoid filtering them out in `is_valid_header` later
-        st.text(alphabet=st.characters(min_codepoint=0, max_codepoint=255, blacklist_characters="\n\r")).map(
-            "Bearer {}".format
-        ),
+        header_value.map("Bearer {}".format),
     )
 
 
@@ -110,7 +119,10 @@ def get_case_strategy(  # pylint: disable=too-many-locals
     The primary purpose of this behavior is to prevent sending incomplete explicit examples by generating missing parts
     as it works with `body`.
     """
-    to_strategy = {DataGenerationMethod.positive: make_positive_strategy}[data_generation_method]
+    to_strategy = {
+        DataGenerationMethod.positive: make_positive_strategy,
+        DataGenerationMethod.negative: make_negative_strategy,
+    }[data_generation_method]
 
     context = HookContext(operation)
 
@@ -124,7 +136,7 @@ def get_case_strategy(  # pylint: disable=too-many-locals
         if body is NOT_SET:
             if operation.body:
                 parameter = draw(st.sampled_from(operation.body.items))
-                strategy = _get_body_strategy(parameter, to_strategy, operation.schema)
+                strategy = _get_body_strategy(parameter, to_strategy, operation)
                 strategy = apply_hooks(operation, context, hooks, strategy, "body")
                 media_type = parameter.media_type
                 body = draw(strategy)
@@ -147,6 +159,7 @@ def get_case_strategy(  # pylint: disable=too-many-locals
         cookies=cookies,
         query=query,
         body=body,
+        data_generation_method=data_generation_method,
     )
 
 
@@ -186,15 +199,17 @@ _BODY_STRATEGIES_CACHE: WeakKeyDictionary = WeakKeyDictionary()
 
 
 def _get_body_strategy(
-    parameter: OpenAPIParameter, to_strategy: Callable[[Dict[str, Any]], st.SearchStrategy], parent_schema: BaseSchema
+    parameter: OpenAPIBody,
+    to_strategy: StrategyFactory,
+    operation: APIOperation,
 ) -> st.SearchStrategy:
     # The cache key relies on object ids, which means that the parameter should not be mutated
     # Note, the parent schema is not included as each parameter belong only to one schema
     if parameter in _BODY_STRATEGIES_CACHE and to_strategy in _BODY_STRATEGIES_CACHE[parameter]:
         return _BODY_STRATEGIES_CACHE[parameter][to_strategy]
     schema = parameter.as_json_schema()
-    schema = parent_schema.prepare_schema(schema)
-    strategy = to_strategy(schema)
+    schema = operation.schema.prepare_schema(schema)
+    strategy = to_strategy(schema, operation.verbose_name, "body", parameter.media_type)
     if not parameter.is_required:
         strategy |= st.just(NOT_SET)
     _BODY_STRATEGIES_CACHE.setdefault(parameter, {})[to_strategy] = strategy
@@ -208,7 +223,7 @@ def get_parameters_value(
     operation: APIOperation,
     context: HookContext,
     hooks: Optional[HookDispatcher],
-    to_strategy: Callable[[Dict[str, Any]], st.SearchStrategy],
+    to_strategy: StrategyFactory,
 ) -> Dict[str, Any]:
     """Get the final value for the specified location.
 
@@ -231,7 +246,7 @@ _PARAMETER_STRATEGIES_CACHE: WeakKeyDictionary = WeakKeyDictionary()
 
 def get_parameters_strategy(
     operation: APIOperation,
-    to_strategy: Callable[[Dict[str, Any]], st.SearchStrategy],
+    to_strategy: StrategyFactory,
     location: str,
     exclude: Iterable[str] = (),
 ) -> st.SearchStrategy:
@@ -255,7 +270,7 @@ def get_parameters_strategy(
             schema["properties"].pop(name, None)
             with suppress(ValueError):
                 schema["required"].remove(name)
-        strategy = to_strategy(schema)
+        strategy = to_strategy(schema, operation.verbose_name, location, None)
         serialize = operation.get_parameter_serializer(location)
         if serialize is not None:
             strategy = strategy.map(serialize)
@@ -266,19 +281,61 @@ def get_parameters_strategy(
             "query": is_valid_query,
         }[location]
         strategy = strategy.filter(filter_func)
-        map_func = {"path": quote_all}.get(location)
+        # Path & query parameters will be cast to string anyway, but having their JSON equivalents for
+        # `True` / `False` / `None` improves chances of them passing validation in apps that expect boolean / null types
+        # and not aware of Python-specific representation of those types
+        map_func = {
+            "path": compose(quote_all, jsonify_python_specific_types),
+            "query": jsonify_python_specific_types,
+        }.get(location)
         if map_func:
-            strategy = strategy.map(map_func)
+            strategy = strategy.map(map_func)  # type: ignore
         _PARAMETER_STRATEGIES_CACHE.setdefault(operation, {})[nested_cache_key] = strategy
         return strategy
     # No parameters defined for this location
     return st.none()
 
 
-def make_positive_strategy(schema: Dict[str, Any]) -> st.SearchStrategy:
+def _jsonify_leaves(value: Any) -> Any:
+    if isinstance(value, dict):
+        for key, sub_item in value.items():
+            value[key] = _jsonify_leaves(sub_item)
+    elif isinstance(value, list):
+        value = [_jsonify_leaves(sub_item) for sub_item in value]
+    elif isinstance(value, bool):
+        return "true" if value else "false"
+    elif value is None:
+        return "null"
+    return value
+
+
+def jsonify_python_specific_types(value: Dict[str, Any]) -> Dict[str, Any]:
+    """Convert Python-specific values to their JSON equivalents."""
+    return _jsonify_leaves(value)
+
+
+def make_positive_strategy(
+    schema: Dict[str, Any], operation_name: str, location: str, media_type: Optional[str]
+) -> st.SearchStrategy:
+    """Strategy for generating values that fit the schema."""
+    if is_header_location(location):
+        # We try to enforce the right header values via "format"
+        # This way, only allowed values will be used during data generation, which reduces the amount of filtering later
+        # If a property schema contains `pattern` it leads to heavy filtering and worse performance - therefore, skip it
+        for sub_schema in schema.get("properties", {}).values():
+            if len(sub_schema) == 1 and "type" in sub_schema:
+                sub_schema.setdefault("format", "_header_value")
     return from_schema(schema, custom_formats=STRING_FORMATS)
 
 
+def make_negative_strategy(
+    schema: Dict[str, Any], operation_name: str, location: str, media_type: Optional[str]
+) -> st.SearchStrategy:
+    return negative_schema(
+        schema, operation_name=operation_name, location=location, media_type=media_type, custom_formats=STRING_FORMATS
+    )
+
+
 def is_valid_path(parameters: Dict[str, Any]) -> bool:
     """Empty strings ("") are excluded from path by urllib3.
 
@@ -289,7 +346,6 @@ def is_valid_path(parameters: Dict[str, Any]) -> bool:
     In this case one variable in the path template will be empty, which will lead to 404 in most of the cases.
     Because of it this case doesn't bring much value and might lead to false positives results of Schemathesis runs.
     """
-
     disallowed_values = (SLASH, "")
 
     return not any(
