@@ -125,15 +125,31 @@ def get_static_parameters_from_properties(operation: APIOperation) -> Dict[str,
 def get_strategies_from_examples(
     operation: APIOperation, examples_field: str = "examples"
 ) -> List[SearchStrategy[Case]]:
+    maps = {}
+    for location, container in LOCATION_TO_CONTAINER.items():
+        serializer = operation.get_parameter_serializer(location)
+        if serializer is not None:
+            maps[container] = serializer
+
+    def serialize_components(case: Case) -> Case:
+        """Applies special serialization rules for case components.
+
+        For example, here, query parameters will be rendered in the `deepObject` style if needed.
+        """
+        for container, map_func in maps.items():
+            value = getattr(case, container)
+            setattr(case, container, map_func(value))
+        return case
+
     strategies = [
-        get_case_strategy(operation=operation, **static_parameters)
+        get_case_strategy(operation=operation, **static_parameters).map(serialize_components)
         for static_parameters in get_static_parameters_from_examples(operation, examples_field)
         if static_parameters
     ]
     for static_parameters in static_parameters_union(
         get_static_parameters_from_example(operation), get_static_parameters_from_properties(operation)
     ):
-        strategies.append(get_case_strategy(operation=operation, **static_parameters))
+        strategies.append(get_case_strategy(operation=operation, **static_parameters).map(serialize_components))
     return strategies
 
 
