@@ -12,6 +12,7 @@ import jsonschema
 import requests
 from hypothesis.strategies import SearchStrategy
 
+from ... import failures
 from ...constants import DataGenerationMethod
 from ...exceptions import (
     InvalidSchema,
@@ -258,6 +259,7 @@ class BaseOpenAPISchema(BaseSchema):
         security_parameters = self.security.get_security_definitions_as_parameters(
             self.raw_schema, operation, self.resolver, location
         )
+        security_parameters = [item for item in security_parameters if item["in"] == location]
         if security_parameters:
             definitions.extend(security_parameters)
         if definitions:
@@ -379,10 +381,12 @@ class BaseOpenAPISchema(BaseSchema):
             return
         content_type = response.headers.get("Content-Type")
         if content_type is None:
-            media_types = "\n    ".join(self.get_content_types(operation, response))
+            media_types = self.get_content_types(operation, response)
+            formatted_media_types = "\n    ".join(media_types)
             raise get_missing_content_type_error()(
                 "The response is missing the `Content-Type` header. The schema defines the following media types:\n\n"
-                f"    {media_types}"
+                f"    {formatted_media_types}",
+                context=failures.MissingContentType(media_types),
             )
         if not is_json_media_type(content_type):
             return
@@ -395,7 +399,10 @@ class BaseOpenAPISchema(BaseSchema):
             exc_class = get_response_parsing_error(exc)
             payload = get_response_payload(response)
             raise exc_class(
-                f"The received response is not valid JSON:\n\n    {payload}\n\nException: \n\n    {exc}"
+                f"The received response is not valid JSON:\n\n    {payload}\n\nException: \n\n    {exc}",
+                context=failures.JSONDecodeErrorContext(
+                    validation_message=exc.msg, document=exc.doc, position=exc.pos, lineno=exc.lineno, colno=exc.colno
+                ),
             ) from exc
         resolver = ConvertingResolver(self.location or "", self.raw_schema, nullable_name=self.nullable_name)
         with in_scopes(resolver, scopes):
@@ -404,7 +411,14 @@ class BaseOpenAPISchema(BaseSchema):
             except jsonschema.ValidationError as exc:
                 exc_class = get_schema_validation_error(exc)
                 raise exc_class(
-                    f"The received response does not conform to the defined schema!\n\nDetails: \n\n{exc}"
+                    f"The received response does not conform to the defined schema!\n\nDetails: \n\n{exc}",
+                    context=failures.ValidationErrorContext(
+                        validation_message=exc.message,
+                        schema_path=list(exc.absolute_schema_path),
+                        schema=exc.schema,
+                        instance_path=list(exc.absolute_path),
+                        instance=exc.instance,
+                    ),
                 ) from exc
         return None  # explicitly return None for mypy
 
@@ -615,7 +629,8 @@ class OpenApi30(SwaggerV20):  # pylint: disable=too-many-ancestors
         scopes, definition = self.resolver.resolve_in_scope(deepcopy(definition), scope)
         options = iter(definition.get("content", {}).values())
         option = next(options, None)
-        if option:
+        # "schema" is an optional key in the `MediaType` object
+        if option and "schema" in option:
             # Extra conversion to JSON Schema is needed here if there was one $ref in the input
             # because it is not converted
             return scopes, to_json_schema_recursive(option["schema"], self.nullable_name)
